package main

import (
    "fmt"
    "io/ioutil"
    "os"
    "os/exec"
    "regexp"
    "strings"
    "flag"
    "log"
    "runtime"
)


type CommitType string

const (
    FEAT     CommitType = "feat"
    FIX      CommitType = "fix"
    DOCS     CommitType = "docs"
    STYLE    CommitType = "style"
    REFACTOR CommitType = "refactor"
    TEST     CommitType = "test"
    CHORE    CommitType = "chore"
    PERF     CommitType = "perf"
    HOTFIX   CommitType = "hotfix"
    MERGE    CommitType = "merge"
)
const CommitMessagePattern = `^(?:fixup!\s*)?(\w*)\:(\[(.*)\])?\:(.*)|^Merge(.*)`
// const CommitMessagePattern = `^(?:fixup!\s*)?(\w*)\:(\[([A-Z]){1,30}([0-9]){1,30}([-])([0-9]){1,30}\])?\:(.*)|^Merge\ branch(.*)`
// const CommitMessagePattern = `^(?:fixup!\s*)?(\w*)\:(\[([\w\$\.\*/-].*)\])?\:(.*)|^Merge\ branch(.*)`

const checkFailedMeassge = `##############################################################################
##                                                                          ##
## 提交信息首行请按照如下格式书写:                                              ##
##                                                                          ##
## commitType:[jira_id]:description\n                                       ##
##   1. commitType只能是feat,fix,docs,style,refactor,test,chore,merge        ##
##   2. jira_id jira id号                                                   ##
##   3. description 没有限制                                                 ##
## Example:                                                                 ##
##   例如：feat:[BC-13656]:domain_info支持域名通配符                           ##
##                                                                          ##
##############################################################################`

// 是否开启严格模式，严格模式下将校验所有的提交信息格式(多 commit 下)
const strictMode = false

var commitMsgReg = regexp.MustCompile(CommitMessagePattern)


var (
    logFileName = flag.String("log", "cServer.log", "Log file name")
)


func main() {

    input, _ := ioutil.ReadAll(os.Stdin)
    param := strings.Fields(string(input))

    runtime.GOMAXPROCS(runtime.NumCPU())
    flag.Parse()
 
    //set logfile Stdout
    logFile, logErr := os.OpenFile(*logFileName, os.O_CREATE|os.O_RDWR|os.O_APPEND, 0666)
    if logErr != nil {
        fmt.Println("Fail to find", *logFile, "cServer start Failed")
        os.Exit(1)
    }
    log.SetOutput(logFile)
    log.SetFlags(log.Ldate | log.Ltime | log.Lshortfile)
 
    //write log
    log.Println(param)
 

    // allow branch/tag delete
    if param[1] == "0000000000000000000000000000000000000000" {
        os.Exit(0)
    }


    if param[0] == "0000000000000000000000000000000000000000" {
        os.Exit(0)
    }
    commitMsg := getCommitMsg(param[0], param[1])
    for _, tmpStr := range commitMsg {
        commitTypes := commitMsgReg.FindAllStringSubmatch(tmpStr, -1)

        if len(commitTypes) != 1 {
            checkFailed()
        } else {
            switch commitTypes[0][1] {
            case string(FEAT):
            case string(FIX):
            case string(DOCS):
            case string(STYLE):
            case string(REFACTOR):
            case string(TEST):
            case string(CHORE):
            case string(PERF):
            case string(HOTFIX):
            case string(MERGE):
            default:
                if !strings.HasPrefix(tmpStr, "Merge") {
                    checkFailed()
                }
            }
        }
        if !strictMode {
            os.Exit(0)
        }
    }


}

func getCommitMsg(odlCommitID, commitID string) []string {
    getCommitMsgCmd := exec.Command("git", "log", odlCommitID+".."+commitID, "--pretty=format:%s")
    getCommitMsgCmd.Stdin = os.Stdin
    getCommitMsgCmd.Stderr = os.Stderr
    b, err := getCommitMsgCmd.Output()
    if err != nil {
        fmt.Print(err)
        os.Exit(1)
    }

    commitMsg := strings.Split(string(b), "\n")
    return commitMsg
}

func checkFailed() {
    fmt.Fprintln(os.Stderr, checkFailedMeassge)
    os.Exit(1)
}

